from begoneads.hostsmanager import HostsManager, start_marker, end_marker
from unittest.mock import patch, mock_open

without_begoneads = '''
127.0.0.1 localhost whatever-hostname
0.0.0.0 some.local
'''

with_begoneads = f'''
127.0.0.1 localhost whatever-hostname
0.0.0.0 some.local

{start_marker}
0.0.0.0 inserted.by.begoneads
{end_marker}
'''


def test_has_begoneads():
    with patch('builtins.open', mock_open(read_data=without_begoneads)) as mock_file:
        hosts_manager = HostsManager('/etc/hosts')
        mock_file.assert_called_with('/etc/hosts', 'r')

        assert hosts_manager.has_begoneads() == None

    with patch('builtins.open', mock_open(read_data=with_begoneads)) as mock_file:
        hosts_manager = HostsManager('/etc/hosts')
        mock_file.assert_called_with('/etc/hosts', 'r')

        assert str(hosts_manager.has_begoneads()).startswith('<re.Match')


def test_apply_hosts():
    with patch('builtins.open', mock_open(read_data=without_begoneads)) as mock_file:
        hosts_manager = HostsManager('/etc/hosts')
        mock_file.assert_called_with('/etc/hosts', 'r')

        assert hosts_manager.has_begoneads() == None

        hosts_manager.apply_hosts('0.0.0.0 some.test')

        assert str(hosts_manager.has_begoneads()).startswith('<re.Match')


def test_remove_begoneads():
    with patch('builtins.open', mock_open(read_data=with_begoneads)) as mock_file:
        hosts_manager = HostsManager('/etc/hosts')
        mock_file.assert_called_with('/etc/hosts', 'r')

        assert str(hosts_manager.has_begoneads()).startswith('<re.Match')

        hosts_manager.remove_begoneads()

        assert hosts_manager.has_begoneads() == None



#!/usr/bin/env python3

import os
import sys
import re
import click
from begoneads.collectors.remote_collector import RemoteCollector
from begoneads.collectors.local_collector import LocalCollector
from begoneads.hostsmanager import HostsManager
from begoneads.exceptions import *
from begoneads.helpers import is_admin

# Default sources
sources = [
    'https://www.malwaredomainlist.com/hostslist/hosts.txt',
    'https://raw.githubusercontent.com/FadeMind/hosts.extras/master/add.Dead/hosts',
    'https://raw.githubusercontent.com/FadeMind/hosts.extras/master/add.Spam/hosts',
    'https://someonewhocares.org/hosts/zero/hosts',
    'http://winhelp2002.mvps.org/hosts.txt',
    'https://pgl.yoyo.org/adservers/serverlist.php?hostformat=hosts&mimetype=plaintext&useip=0.0.0.0',
    'https://raw.githubusercontent.com/mitchellkrogza/Badd-Boyz-Hosts/master/hosts',
    'https://zerodot1.gitlab.io/CoinBlockerLists/hosts_browser',
    'https://raw.githubusercontent.com/FadeMind/hosts.extras/master/UncheckyAds/hosts',
    'https://raw.githubusercontent.com/FadeMind/hosts.extras/master/add.2o7Net/hosts',
    'https://raw.githubusercontent.com/azet12/KADhosts/master/KADhosts.txt',
    'https://raw.githubusercontent.com/AdAway/adaway.github.io/master/hosts.txt',
    'https://raw.githubusercontent.com/FadeMind/hosts.extras/master/add.Risk/hosts',
]


@click.group()
def cli():
    """Install or uninstall BeGoneAds, the host blocker for the system hosts file"""


@cli.command('install', short_help='Install or update BeGoneAds')
@click.option('--sources', is_flag=False, default=','.join(sources),
              type=click.STRING, help='Sets sources to fetch from, seperated by ,')
@click.option('--local-sources', is_flag=False, default='',
              type=click.STRING, help='Pass local hosts files to include')
def install(sources, local_sources):
    # Check if we have sufficient permissions
    if not is_admin(sys.platform.startswith('win')):
        raise NotElevatedException(
            'This program needs to be run as root to work properly')

    sources = [i.strip() for i in sources.split(',')]
    local_sources = [i.strip() for i in local_sources.split(',')]

    url_pattern = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        # domain...
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE
    )

    for source in sources:
        if not re.match(url_pattern, source):
            raise InvalidSourceException(source)

    for source in local_sources:
        if not os.path.isfile(source):
            raise InvalidSourceException(source)

    # Collect hosts for hosts file
    remote_collector = RemoteCollector(sources)

    print('⋯ Collecting and parsing remote hosts')
    remote_hosts = remote_collector.get_result()

    print('✓ Remote hosts collected')

    # Collect local host files
    local_collector = LocalCollector(local_sources)

    print('⋯ Collecting and parsing local hosts')
    local_hosts = local_collector.get_result()

    hosts = "\n".join([
        remote_hosts,
        local_hosts
    ])

    print('✓ Local hosts collected')

    if sys.platform.startswith('win'):
        path = r'c:\windows\system32\drivers\etc\hosts'
    else:
        path = '/etc/hosts'

    print('⋯ Parse current hosts file')
    hosts_manager = HostsManager(path)

    print('⋯ Applying new hosts')
    hosts_manager.apply_hosts(hosts)

    print('⋯ Saving')
    hosts_manager.commit()

    print('✓ Hosts applied')


@cli.command('uninstall', short_help='Uninstall BeGoneAds')
def uninstall():
    # Check if we have sufficient permissions
    if not is_admin(sys.platform.startswith('win')):
        raise NotElevatedException(
            'This program needs to be run as root to work properly')

    if sys.platform.startswith('win'):
        path = r'c:\windows\system32\drivers\etc\hosts'
    else:
        path = '/etc/hosts'

    print('⋯ Parse current hosts file')
    hosts_manager = HostsManager(path)

    print('⋯ Removing BeGoneAds')
    hosts_manager.remove_begoneads()

    print('⋯ Saving')
    hosts_manager.commit()

    print('✓ BeGoneAds uninstalled')


if __name__ == '__main__':
    cli()



import requests
import re
from tqdm import tqdm
from begoneads.collectors.base_collector import BaseCollector


class RemoteCollector(BaseCollector):
    """A class that collects the remote host files

    Attributes:
        sources: list
    """

    def try_get_sources(self, sources):
        """Try and get each source, don't return them if the request was not succesful"""

        filtered = []

        for source in tqdm(sources):
            response = requests.get(source)

            if response.status_code >= 200 and response.status_code < 300:
                content = str(response.text)

                filtered.append(content)

        return filtered



import requests
import re
from tqdm import tqdm
from begoneads.collectors.base_collector import BaseCollector


class LocalCollector(BaseCollector):
    """A class that collects local host files

    Attributes:
        sources: list
    """

    def try_get_sources(self, sources):
        """Try and get each file"""

        filtered = []

        for source in tqdm(sources):
            with open(source) as _file:
                filtered.append(_file.read())

        return filtered



class NotElevatedException(Exception):
    pass


class InvalidSourceException(Exception):
    def __init__(self, source):
        self.message = f'{source} is not a valid source'

    def __str__(self):
        return self.message



import requests
import re
from tqdm import tqdm


class BaseCollector(object):
    """A base class for all collectors

    Attributes:
        sources: list
    """

    def __init__(self, sources):
        self.sources = sources

    def try_get_sources(self, sources):
        """Try and get each file"""

        filtered = []

        for source in tqdm(sources):
            with open(source) as _file:
                filtered.append(_file.read())

        return filtered

    def fix_ips(self, hosts):
        """Replace all IP addresses with 0.0.0.0"""

        hosts = re.sub(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', '0.0.0.0', hosts, 0, re.MULTILINE)

        return hosts

    def filter_hosts(self, hosts):
        """Only keep meaningful lines"""

        hosts = hosts.split('\n')
        hosts = list(set(hosts))

        hosts = [line for line in hosts
                 if line.strip() != '' and not line.startswith('#')]
        hosts = '\n'.join(hosts)

        return hosts

    def get_result(self):
        """Get usable result"""

        sources = self.try_get_sources(self.sources)
        sources = '\n'.join(sources)
        hosts = self.fix_ips(sources)
        hosts = self.filter_hosts(hosts)

        return hosts



from begoneads.collectors.remote_collector import RemoteCollector
from unittest.mock import patch

collector = RemoteCollector([])


def test_try_get_sources():
    with patch('requests.get') as mock_request:
        url = 'https://somewhere.com/that/has/hosts'

        mock_request.return_value.status_code = 200
        mock_request.return_value.text = 'response'

        assert collector.try_get_sources([url]) == ['response']
        mock_request.assert_called_once_with(url)

    with patch('requests.get') as mock_request:
        url = 'https://somewhere.com/that/has/hosts/but/with/error'

        mock_request.return_value.status_code = 404

        assert collector.try_get_sources([url]) == []
        mock_request.assert_called_once_with(url)


def test_fix_ips():
    before = '127.0.0.1 some.host'

    result = collector.fix_ips(before)

    assert result == '0.0.0.0 some.host'



import os


def is_admin(is_windows):
    if is_windows:
        try:
            # only windows users with admin privileges can read the C:\windows\temp
            temp = os.listdir(os.sep.join(
                [os.environ.get('SystemRoot', 'C:\\windows'), 'temp']))
        except:
            return False
        else:
            return True
    else:
        if os.geteuid() == 0:
            return True
        else:
            return False



import re


start_marker = '### START BeGoneAds ###'
end_marker = '### END BeGoneAds ###'
pattern = re.compile(f'(?<={start_marker})(.*)(?={end_marker})',
                     re.MULTILINE | re.DOTALL)
fullpattern = re.compile(f'(\n\n{start_marker}.*{end_marker})',
                         re.MULTILINE | re.DOTALL)


class HostsManager(object):
    """Manager for the system hosts file"""

    def __init__(self, path):
        self.path = path

        self.get_content()

    def get_content(self):
        """Get current contents of file"""

        file = open(self.path, 'r', encoding='utf-8')
        self.content = file.read()
        file.close()

    def has_begoneads(self):
        """Check if BeGoneAds is installed"""

        return re.search(pattern, self.content)

    def apply_hosts(self, hosts):
        """Apply or append BeGoneAds"""

        if self.has_begoneads():
            self.content = re.sub(pattern, f'\n{hosts}\n', self.content)
        else:
            self.content = f'{self.content}\n\n{start_marker}\n{hosts}\n{end_marker}\n'

    def remove_begoneads(self):
        """Remove BeGoneAds"""

        if self.has_begoneads():
            self.content = re.sub(fullpattern, '', self.content)

    def commit(self):
        """Save hosts file"""

        file = open(self.path, 'w', encoding='utf-8')
        file.write(self.content.replace('\r', ''))
        file.close()



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='begoneads',
    version='0.0.5',
    description='BeGoneAds puts some popular hosts file lists into the hosts file as a adblocker measure.',
    long_description=readme(),
    long_description_content_type="text/markdown",
    url='http://github.com/anned20/begoneads',
    entry_points={
        'console_scripts': ['begoneads=begoneads.begoneads:cli'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: System :: Networking',
    ],
    author='Anne Douwe Bouma',
    author_email='annedouwe@bouma.tech',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'click',
        'requests',
        'tqdm',
    ],
    zip_safe=False
)
